﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;
using SamplePortableClassLibrary.Models;
using System.Windows.Input;
using System.Net.Http;
using System.Threading.Tasks;
using SamplePortableClassLibrary.Functions;

namespace SamplePortableClassLibrary.ViewModels
{
    public class PersonViewModel : INotifyPropertyChanged 
    {
        /// <summary>
        /// 실버라이트를 제거한 후에만 가능
        /// </summary>
        HttpClient client;

        public event PropertyChangedEventHandler PropertyChanged;
        protected void RaisePropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        ObservableCollection<Person> people;
        public ObservableCollection<Person> People 
        {
            get { return people; }
            set 
            {
                people = value;
                RaisePropertyChanged("People");
            }
        }

        private Person currentPerson;

        public Person CurrentPerson
        {
            get { return currentPerson; }
            set 
            { 
                currentPerson = value;
                RaisePropertyChanged("CurrentPerson");
            }
        }

        public PersonViewModel()
        {
            //httpClient 초기화
            client = new HttpClient();

            //People = new ObservableCollection<Person>
            //{
            //    new Person{ Id=0 , Age=11 , Name="kaki104" , Sex=true},
            //    new Person{ Id=1 , Age=22 , Name="kakao" , Sex=false},
            //    new Person{ Id=2 , Age=33 , Name="kass" , Sex=true},
            //    new Person{ Id=3 , Age=44 , Name="kameo" , Sex=false},
            //    new Person{ Id=4 , Age=55 , Name="keroro" , Sex=true}
            //};
            //CurrentPerson = People.FirstOrDefault();

            getAllData();
        }

        ICommand addCommand;
        public ICommand AddCommand 
        {
            get 
            {
                if (addCommand == null)
                {
                    addCommand = new DelegateCommand(obj => 
                    {
                        CurrentPerson = new Person();
                        //서버에서 Id를 자동으로 생성하기 때문에 여기는 막음
                        //CurrentPerson.Id = People.Max(p => p.Id) + 1;
                        People.Add(CurrentPerson);
                        string url = "http://localhost:61466/api/personContainer/InsertPerson";
                        SPA_crud_Operation(url, CurrentPerson);
                    });
                }
                return addCommand; 
            }
        }

        ICommand removeCommand;
        public ICommand RemoveCommand
        {
            get
            {
                if (removeCommand == null)
                {
                    removeCommand = new DelegateCommand(obj =>
                    {
                        var person = obj as Person;
                        if (person != null && People.Count(p => p.Id == person.Id) > 0)
                        {
                            People.Remove(person);
                            string url = "http://localhost:61466/api/personContainer/DeletePerson";
                            SPA_crud_Operation(url, person);
                        }
                    });
                }
                return removeCommand;
            }
        }

        ICommand selectChangedCommand;
        public ICommand SelectChangedCommand
        {
            get
            {
                if (selectChangedCommand == null)
                {
                    selectChangedCommand = new DelegateCommand(obj =>
                    {
                        var person = obj as Person;
                        if (person != null && People.Count(p => p.Id == person.Id) > 0)
                        {
                            if (CurrentPerson != null)
                            {
                                string url = "http://localhost:61466/api/personContainer/UpdatePerson";
                                SPA_crud_Operation(url, CurrentPerson);
                            }

                            CurrentPerson = person;
                        }
                    });
                }
                return selectChangedCommand;
            }
        }

        //ICommand getCommand;
        //public ICommand GetCommand
        //{
        //    get
        //    {
        //        if (getCommand == null)
        //        {
        //            getCommand = new DelegateCommand(async obj =>
        //            {
        //                Person per = obj as Person;
        //                HttpResponseMessage response;
        //                if (per == null)
        //                {
        //                    getAllData();
        //                }
        //                else
        //                { 
        //                }

        //            });
        //        }
        //        return getCommand;
        //    }
        //}

        private async void getAllData()
        {
            //전체 데이터 조회
            var response = await client.GetAsync("http://localhost:61466/api/personContainer/GetPeople");
            string content = await response.Content.ReadAsStringAsync();
            var people = StaticFunctions.ReadToObject(typeof(ObservableCollection<Person>), content) as ObservableCollection<Person>;
            People = new ObservableCollection<Person>(people);
        }

        /// <summary>
        /// Single Page Application에 있는 WebAPI를 호출해서 CRUD 작업
        /// </summary>
        /// <param name="url"></param>
        /// <param name="person"></param>
        private async void SPA_crud_Operation(string url, Person person)
        {
            var response = await client.PostAsync(url,
                                    new StringContent(StaticFunctions.WriteFromObject(typeof(Person), person),
                                    Encoding.UTF8,
                                    "application/json"));

            string content = await response.Content.ReadAsStringAsync();
            var resultPerson = StaticFunctions.ReadToObject(typeof(Person), content) as Person;
            if (resultPerson != null)
            {
                person.Id = resultPerson.Id;
                person.Age = resultPerson.Age;
                person.Name = resultPerson.Name;
                person.Sex = resultPerson.Sex;
            }
        }

    }
}
